// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef CMAKE_CONFIG_CONFIG_H
#define CMAKE_CONFIG_CONFIG_H

#include <array>

#define DRAWPILE_SOURCE_ASSETS_DESKTOP "/home/iurt/rpmbuild/BUILD/drawpile-2.2.2-build/Drawpile-2.2.2/src/desktop/assets"

// The version number goes to a separate TU because it will change after every
// commit, and so if it were in this header that would a bunch of other TUs to
// invalidate on every new commit which would be bad for build speed. It's
// extern "C" so that it can be called reasonably easily from Rust.
extern "C" const char *DP_cmake_config_version();

namespace cmake_config {

inline const char *version() { return DP_cmake_config_version(); }
constexpr const char *website() { return "https://drawpile.net"; }
constexpr auto locales() { return std::array{ "ar_EG", "cs_CZ", "de_DE", "en_US", "es_CO", "fi_FI", "fr_FR", "id_ID", "it_IT", "ja_JP", "ko_KR", "pl_PL", "pt_BR", "pt_PT", "ru_RU", "th_TH", "tr_TR", "uk_UA", "vi_VN", "zh_CN" }; }

#ifdef DRAWPILE_SOURCE_ASSETS_DESKTOP
constexpr const char *sourceAssetsDesktop() { return DRAWPILE_SOURCE_ASSETS_DESKTOP; }
#endif

namespace file_group {
	constexpr auto canvasReplacing() { return std::array{ "dprec", "dptxt", "ora", "psd"  }; }
	constexpr const char *image() { return "*.ora *.psd *.png *.jpg *.jpeg *.webp"; }
	constexpr const char *layeredImage() { return "*.ora *.psd"; }
	constexpr const char *flatImage() { return "*.png *.jpg *.jpeg *.webp"; }
	constexpr const char *recording() { return "*.dprec *.dptxt"; }
} // namespace file_group

namespace proto {
constexpr int server() { return 4; }
constexpr int major() { return 24; }
constexpr int minor() { return 0; }
constexpr int port() { return 27750; }
} // namespace proto

} // namespace config

#endif
